precision highp float;
varying vec3 v_position;

uniform samplerCube cubeTexture;

const float PI = 3.14159265359;

vec3 calculateSample(float sinPhi, float cosPhi, float sinTheta, float cosTheta, vec3 up, vec3 right, vec3 N)
{
    // spherical to cartesian (in tangent space)
    vec3 tangentSample = vec3(sinTheta * cosPhi,  sinTheta * sinPhi, cosTheta);
    // tangent space to world
    vec3 sampleVec = tangentSample.x * right + tangentSample.y * up + tangentSample.z * N; 

    return textureCube(cubeTexture, sampleVec, 0.0).rgb * cosTheta * sinTheta;
}

vec3 calculateIrradiance(float sinPhi, float cosPhi, vec3 up, vec3 right, vec3 N)
{
    // float theta = 0.0;
    // float thetaDelta = 0.0981747;
    vec3 irradiance = vec3(0.0);
    irradiance += calculateSample(sinPhi, cosPhi, 0.0, 0.1, up, right, N); // sin and cos of theta
    irradiance += calculateSample(sinPhi, cosPhi, 0.09801707024, 0.9951847336, up, right, N); // sin and cos of thetaDelta
    irradiance += calculateSample(sinPhi, cosPhi, 0.1950901839, 0.9807853079, up, right, N); // sin and cos of thetaDelta * 2
    irradiance += calculateSample(sinPhi, cosPhi, 0.2902844751, 0.9569403971, up, right, N); // sin and cos of thetaDelta * 3
    irradiance += calculateSample(sinPhi, cosPhi, 0.3826831721, 0.9238796403, up, right, N); // sin and cos of thetaDelta * 4
    irradiance += calculateSample(sinPhi, cosPhi, 0.4713964263, 0.8819214303, up, right, N); // sin and cos of thetaDelta * 5
    irradiance += calculateSample(sinPhi, cosPhi, 0.5555698817, 0.8314698471, up, right, N); // sin and cos of thetaDelta * 6
    irradiance += calculateSample(sinPhi, cosPhi, 0.6343929031, 0.7730107661, up, right, N); // sin and cos of thetaDelta * 7

    return irradiance;
}

void main()
{		
	// The world vector acts as the normal of a tangent surface
    // from the origin, aligned to v_position. Given this normal, calculate all
    // incoming radiance of the environment. The result of this radiance
    // is the radiance of light coming from -Normal direction, which is what
    // we use in the PBR shader to sample irradiance.
    vec3 N = normalize(v_position);   
    
    // tangent space calculation from origin point
    vec3 up    = vec3(0.0, 1.0, 0.0);
    vec3 right = cross(up, N);
    up            = cross(N, right);

    vec3 irradiance = calculateIrradiance(0.0, 1.0, up, right, N);
    irradiance += calculateIrradiance(0.7071067811, 0.7071067813, up, right, N); // sin and cos of phiDelta
    irradiance += calculateIrradiance(1.0, 0.0, up, right, N); // sin and cos of phiDelta * 2
    irradiance += calculateIrradiance(0.7071067814, -0.707106781, up, right, N); // sin and cos of phiDelta * 3
    irradiance += calculateIrradiance(0.0, -1.0, up, right, N); // sin and cos of phiDelta * 4
    irradiance += calculateIrradiance(-0.7071067808, -0.7071067815, up, right, N); // sin and cos of phiDelta * 5
    irradiance += calculateIrradiance(-1.0, 0.0, up, right, N); // sin and cos of phiDelta * 6
    irradiance += calculateIrradiance(-0.7071067817, 0.7071067807, up, right, N); // sin and cos of phiDelta * 7

    irradiance *= 0.049087384; // 0.049087384 = PI * (1.0 / 64.0);
    
    gl_FragColor = vec4(irradiance, 1.0);
}